/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2007 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <pthread.h>
#include <signal.h>
#include <sys/types.h>

#include "String.h"

#define message(msglevel, args...)	if (msglevel <= Options.DebugLvl) fprintf(stderr, args)

#ifdef DEBUG
#define assert(condition)			if (!(condition)) { fprintf(stderr, "Assertion failed.\n"); abort(); }
#else
#define assert(condition)
#endif

#ifndef DEFAULT_SMBCLIENT
#define DEFAULT_SMBCLIENT	"/usr/bin/smbclient"
#endif

#ifndef DEFAULT_NMBLOOKUP
#define DEFAULT_NMBLOOKUP	"/usr/bin/nmblookup"
#endif

#ifndef DEFAULT_SMBLISTER
#define DEFAULT_SMBLISTER	"/usr/bin/smblister"
#endif

#ifndef LOCALEDIR
#define LOCALEDIR			"/usr/share/locale"
#endif

#define IP_ADDRESS(x)			((x & 0xff000000) >> 24), ((x & 0x00ff0000) >> 16), ((x & 0x0000ff00) >> 8), ((x & 0x000000ff) >> 0)

struct Options_Structure {
	double Timings;
	char Workgroup[256];
	int DebugLvl;
	char NmbLookup_Executable[256];
	char SmbClient_Executable[256];
	char SmbLister_Executable[256];
	char Outfile[256];
	int ThreadCount;
	int PreserveTemp;
	int TryPasswords;
	int Filter_PrintShares;
	int Connect_By_IP;
	int Secondary_Lookup;
	StringList UserPass;
	char HostlistFile[256];
	char SharelistFile[256];
};

#define TEMPPATH					"/tmp/"
#define FILE_WORKGROUPINFO			(TEMPPATH "sambascanner_workgroup.tmp")
#define FILE_MASTERBROWSER			(TEMPPATH "sambascanner_masterbrowser.tmp")

#define BUFSIZE_SHORT				256
#define BUFSIZE						1024

#define TID(x)						((unsigned long)x)
#define	_(x)						gettext(x)

#define TIMEOUT_MIN					0.1
#define TIMEOUT_MAX					1000.0

/* Timeout to look up workgroup */
/* nmblookup -M -S -- - */
#define TIMEOUT_LOOKUPWG_TERM		10.0
#define TIMEOUT_LOOKUPWG_KILL		5.0

/* Timeout to look up master browser */
/* nmblookup -M -S MORPHEUS */
#define TIMEOUT_LOOKUPMB_TERM		10.0
#define TIMEOUT_LOOKUPMB_KILL		5.0

/* Timeout to process a host, e.g. get the share names the host provides */
/* smbclient -g -N -L //TRINITY */
#define TIMEOUT_PROCESSHOST_TERM	10.0
#define TIMEOUT_PROCESSHOST_KILL	5.0

/* Timeout to retrieve all files of a share */
/* smblister smb://TRINITY/pictures */
#define TIMEOUT_SMBLISTER_TERM		20.0
#define TIMEOUT_SMBLISTER_KILL		5.0

/* Timeout to scan an IP address for its NetBios name */
/* nmblookup -A 192.168.1.50 */
#define TIMEOUT_IPSCAN_TERM			7.5
#define TIMEOUT_IPSCAN_KILL			5.0

