/*
	SambaScanner - A tool to scan Samba networks for files.
	Copyright (C) 2005-2007 Johannes Bauer
	
	This file is part of SambaScanner.

    SambaScanner is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; this program is ONLY licensed under
	version 2 of the License, later versions are explicitly excluded.

    SambaScanner is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with SambaScanner; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include "config.h"

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/wait.h>
#include <string.h>
#include <errno.h>
#include <pthread.h>
#include <libintl.h>
#include <unistd.h>

#include "sambascanner.h"
#include "ProcessTable.h"
#include "SignalHandler.h"
#include "Threads.h"
#include "Mutex.h"

extern struct Options_Structure Options;

void* SignalHandler_Thread(void *Parameter) {
	sigset_t Mask;
	sigemptyset(&Mask);
	sigaddset(&Mask, SIGCHLD);
	message(6, _("Signal handler has been installed: PID %d\n"), getpid());
	while (1) {
		int Signal;
		int CleanedUp = 0;
		sigwait(&Mask, &Signal);
		message(5, _("Signal handler received signal...\n"));
		ProcessTable_Rewind();
		while (ProcessTable_Next() != PROCESSTABLE_END) {
			if (ProcessTable_GetCurrent_Running()) {
				int Status;
				pid_t PID;
				pid_t WaitForPID = ProcessTable_GetCurrent_PID();
				PID = waitpid(WaitForPID, &Status, WNOHANG);
				if (PID == -1) {
					/* Error in waitpid */
					message(4, _("  + waitpid threw error upon waiting on process PID = %d: %s\n"), WaitForPID, strerror(errno));
				} else if (PID > 0) {
					/* Child really exited */
					message(5, _("  - Signal handler found exited process PID = %d.\n"), WaitForPID);
					CleanedUp++;
					ProcessTable_SetCurrent_NotRunning();
					ProcessTable_SetCurrent_Status(Status);
					Signal_Died_Children();
				}
			}
		}
		message(5, _("...signal handler finished checking - %d cleaned up.\n"), CleanedUp);
	}
	pthread_exit(NULL);
	return NULL;
}

void Install_SignalHandler() {
	sigset_t Mask;
	pthread_t SignalHandler;
	
	/* Block SIGCHLD first */
	sigemptyset(&Mask);
	sigaddset(&Mask, SIGCHLD);
	pthread_sigmask(SIG_BLOCK, &Mask, NULL);
	
	/* Now create signal handling thread (it's alright - even neccessary! - that we just blocked signals) */
	pthread_create_detached(&SignalHandler, SignalHandler_Thread, NULL);
}

